-----------------------------------------------------------------------------------------------
-- OhmnaKnockbackAssist
-- an addon to help you with Ohmnas and the Maws knockback mechnic
-- created by Caleb - calebzor@gmail.com
-- slash command to turn the addon on/off: /oka
-----------------------------------------------------------------------------------------------
require "Window"
require "GameLib"
require "GroupLib"
require "CColor"

local GameLib = GameLib
local GroupLib = GroupLib
local Apollo = Apollo
local Vector3 = Vector3
local Vector2 = Vector2
local pairs = pairs
local math = math
local tonumber = tonumber
local string = string
local CColor = CColor
local Print = Print
local tostring = tostring

local function hexToCColor(color, a)
	if not a then a = 1 end
	local r = tonumber(string.sub(color,1,2), 16) / 255
local g = tonumber(string.sub(color,3,4), 16) / 255
	local b = tonumber(string.sub(color,5,6), 16) / 255
	return CColor.new(r,g,b,a)
end

local tColor = {
	yellow = hexToCColor("fff600"),
	orange = hexToCColor("feb408"),
	red = hexToCColor("c6002a"),
	green = hexToCColor("01a825"),
	blue = hexToCColor("00b0d8"),
}

local addon = Apollo.GetPackage("Gemini:Addon-1.1").tPackage:NewAddon("OhmnaKnockbackAssist", false, {}, "Gemini:Timer-1.0")
local t1, t2

local defaults = {
	profile = {
		enabled = true,
	}
}

function addon:OnInitialize()
	self.db = Apollo.GetPackage("Gemini:DB-1.0").tPackage:New(self, defaults)
end

function addon:OnEnable()
	if self.db.profile.enabled then
		self:OnMyEnable()
	end
	Apollo.RegisterSlashCommand("oka", "OnSlash", self)
end

function addon:OnSlash()
	self.db.profile.enabled = not self.db.profile.enabled
	Print("OhmnaKnockbackAssist enabled: "..tostring(self.db.profile.enabled))
	if self.db.profile.enabled then
		self:OnMyEnable()
	end
end

function addon:OnMyEnable()
	self.wMain = Apollo.LoadForm("OhmnaKnockbackAssist.xml", "DebugForm", nil, self)
	--self.testTimer = self:ScheduleRepeatingTimer("OnUpdate", 0.01)
	self:SetupMarks()
	Apollo.RegisterEventHandler("NextFrame", "OnUpdate", self)
	t1 = self.wMain:FindChild("Facing")
	t2 = self.wMain:FindChild("Pos")
	self.tUnits = {}
	--self.tUnits[1] = GameLib.GetPlayerUnit()
	Apollo.RegisterEventHandler("UnitCreated", "OnUnitCreated", self)
	Apollo.RegisterEventHandler("UnitEnteredCombat", "CombatStateChanged", self)
end

function addon:WipeCheck()
	for i=1, GroupLib.GetMemberCount() do
		local unit = GroupLib.GetUnitForGroupMember(i)
		if unit then
			if unit:IsInCombat() then
				return
			end
		end
	end
	self:CancelTimer(self.wipeTimer)
	self.wipeTimer = nil
	self:DestroyMarks()
	self.tUnits = {}
end

function addon:CombatStateChanged(unit, bInCombat)
	if unit == GameLib.GetPlayerUnit() and not bInCombat and not self.wipeTimer then
		self.wipeTimer = self:ScheduleRepeatingTimer("WipeCheck", 0.5)
	end
end

function addon:OnUnitCreated(unit)
	if unit and (unit:GetName() == "Ravenous Maw of the Dreadphage" or unit:GetName() == "Dreadphage Ohmna") and unit:GetType() == "NonPlayer" then
		if not self.tUnits[unit:GetId()] then
			self.tUnits[unit:GetId()] = unit
		end
	end
end

function addon:DestroyMarks()
	if not self.markers then return end
	for i = 1, #self.markers do
		for j = 1, #self.markers[i] do
			self.markers[i][j]:Destroy()
		end
		self.markers[i] = nil
	end
end

function addon:SetupMarks()
	self.markers = {}
	for i = 1, 12 do
		self.markers[i] = {}
		for j = 1, 30 do
			self.markers[i][j] = Apollo.LoadForm("OhmnaKnockbackAssist.xml", "Marker", "InWorldHudStratum", self)
			--if (i % 3) == 0 then
			--	self.markers[i][j]:SetBGColor(tColor.yellow)
			--else
			--	self.markers[i][j]:SetBGColor(tColor.blue)
			--end
			self.markers[i][j]:SetBGColor(tColor.blue)
		end
	end
end

local nLineLength = 30

function addon:OnUpdate()
	if not self.tUnits then return end

	local counter = 0
	for k, v in pairs(self.tUnits) do
		local tFacing = v:GetFacing()
		if not tFacing then return end

		local tPos = v:GetPosition()
		if not tPos then return end
		--t2:FindChild("Pos"):SetText(("y: %s x: %s z: %s"):format(tPos.y, tPos.x, tPos.z)) -- debug

		local rot = math.atan2(tFacing.x, tFacing.z)

		local rotPlus = rot+math.rad(120) -- offset
		local rotNeg = rot+math.rad(-120) -- offset

		tFacing.x = tFacing.x -- because else it is an inverted flipped coordinate system

		if not self.markers[1] or not self.markers[1][1] then self:SetupMarks() end -- check if at least 1 is set up or not

		for nCounterCounter = 1, 3 do -- 3 safe areas per mob
			for i = 1, #self.markers[counter+nCounterCounter] do
				if (nCounterCounter%3) == 1 then
					local tPlusEndPoint = { x = tPos.x+nLineLength*math.sin(rotPlus), y = tPos.y, z = tPos.z+nLineLength*math.cos(rotPlus)}
					local vV1 = Vector3.New(tPos.x, tPos.y, tPos.z)
					local vV2 = Vector3.New(tPlusEndPoint.x, tPlusEndPoint.y, tPlusEndPoint.z)
					local vVector = Vector3.InterpolateLinear(vV1, vV2, (1/#self.markers[counter+nCounterCounter]) * i)
					self.markers[counter+nCounterCounter][i]:SetWorldLocation(vVector)
					self.markers[counter+nCounterCounter][i]:Show(true)
				elseif (nCounterCounter%3) == 2 then
					local tNegEndPoint = { x = tPos.x+nLineLength*math.sin(rotNeg), y = tPos.y, z = tPos.z+nLineLength*math.cos(rotNeg)}
					local vV1 = Vector3.New(tPos.x, tPos.y, tPos.z)
					local vV2 = Vector3.New(tNegEndPoint.x, tNegEndPoint.y, tNegEndPoint.z)
					local vVector = Vector3.InterpolateLinear(vV1, vV2, (1/#self.markers[counter+nCounterCounter]) * i)
					self.markers[counter+nCounterCounter][i]:SetWorldLocation(vVector)
					self.markers[counter+nCounterCounter][i]:Show(true)
				elseif (nCounterCounter%3) == 0 then
					local tHeadEndPoint = { x = tPos.x+nLineLength*math.sin(rot), y = tPos.y, z = tPos.z+nLineLength*math.cos(rot)}
					local vV1 = Vector3.New(tPos.x, tPos.y, tPos.z)
					local vV2 = Vector3.New(tHeadEndPoint.x, tHeadEndPoint.y, tHeadEndPoint.z)
					local vVector = Vector3.InterpolateLinear(vV1, vV2, (1/#self.markers[counter+nCounterCounter]) * i)
					self.markers[counter+nCounterCounter][i]:SetWorldLocation(vVector)
					self.markers[counter+nCounterCounter][i]:Show(true)
					self.markers[counter+nCounterCounter][i]:SetBGColor(tColor.yellow)
				end
			end
		end
		counter = counter + 3
	end
end
